/********************************************************
Workspace: IAR+SPL
Chip type: STM8S003K3 (16MHz HSI)
Project  : thermofan controller
Version  : 0
Date     : Nov 2017
*********************************************************/

#include "stm8s.h"
#include "iostm003.c"
#include "bitset.c"
#include "stdbool.h"
#include "defines.c"
#include "periph_init.c"
#include "kwork.c"
#include "eeprom_routines.c"
#include "ss_display_work.c"
#include "apow_work.c"
 
volatile u16 tim0,tim1,tim2,tim3,tim4;
volatile bool iwdg_t4,iwdg_t2;



extern "C"
{
 INTERRUPT_HANDLER(TIM2_CAPCOM, 14)
 {
  u16 tmp = TIM2->CCR2H<<8;  
      tmp|= TIM2->CCR2L;  
      
  if(tmp<(u16)((u32)16E6/200/16))goto exit_cc2;    
  
  TIM2->EGR = TIM2_EGR_UG;     
  
  TIM2->CCER1^= TIM2_CCER1_CC2P;  
  
  if(dval<0xff) 
  {
   tmp = (u32)tmp*dval>>8; 
   TIM2->CCMR1 = pwm2;
   TIM2->CCMR3 = pwm2;
  }   
  else
  {
   TIM2->CCMR1 = force_inactive;
   TIM2->CCMR3 = force_inactive;   
  }
  
  TIM2->CCR1H = TIM2->CCR3H = tmp>>8;     
  TIM2->CCR1L = TIM2->CCR3L = (u8)tmp;  
  
  PA_ODR^=0xff;
  
  exit_cc2:
  iwdg_t2 = 1;
  TIM2->SR1 = 0;
 }
 
 INTERRUPT_HANDLER(TIM4_OVF, 23)   //1000Hz
 { 
  static u8 cnt;
  if(cnt)cnt--;
  else 
  {
   cnt = 9;
   PF_ODR^=0xff;
  } 
  
  
  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;  
  if(tim3)tim3--;   
  if(tim4)tim4--;    
 
  scan_keys();
  
  iwdg_t4=1;
  TIM4->SR1 = 0;
 } 
}

void devcon(void);

void main()
{
 cfg.gpio();
 cfg.clk();
 cfg.t1(16000);
 cfg.t2();
 cfg.t4();
 cfg.ee();
 
 detect_7seg_polarity(ss_pol); 
 calc_pw();
 
 ITC_SetSoftwarePriority(ITC_IRQ_TIM2_CAPCOM, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_0); 
 
 cfg.iwdg();
 
 asm("rim");

 
 endless
 {
  devcon();
  display_data();

  
  if(iwdg_t2 && iwdg_t4){iwdg_t2=iwdg_t4=0; IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static bool itr=0;
 static enum {start,work,sdn}dev_st = start;
 static bool btn_trg;
 
 if(btn[0]==rls && btn[1]==rls) 
 {
  btn_trg=0;
 }

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;
    dpp[1]=dpp[0]=1;
    tim0=0;
   } 
  
   if(!tim0)   
   { 
    static u8 i=0; 
     
    tim0=100;   
     
    if(!i)       {dbuf[1]=dbuf[0]=10;}   
    else if(i<6) {dbuf[1]++;dbuf[0]++;}   
    else if(i==6){dbuf[1]=dbuf[0]=' ';}  
    else i=0xff, dev_st = work; 
    i++;   
   }   
     
   if(dev_st!=start){itr=0;}
  }
  break;

  case work:
  {
   static u8 dv_tmp[2];

   if(!itr)
   {
    itr=1;
    dv_tmp[1] = read_ee_u8(eep.airflow, sizeof(eep.airflow)); 
    set_airflow(dv_tmp[1]);
    dv_tmp[0] = read_ee_u8(eep.heatpow, sizeof(eep.heatpow));
    set_pow (dv_tmp[0]);
   }    

   
   static bool pb_trg[2];
   
   for(u8 i=0;i<2;i++)
   {
    if(btn[i]==prs)
    {
     pb_trg[i]=1;
    }
    else if(btn[i]==hold)
    {
     pb_trg[i]=0;
     
     if(btn[1]==rls || btn[0]==rls)
     if(!tim0)
     {
      tim0=250;
      if(dv_tmp[i]<9)dv_tmp[i]++;
      else dv_tmp[i]=0;      
     }
    }
    else
    {
     if(pb_trg[i])
     {
      if(dv_tmp[i]<9)dv_tmp[i]++;
      else dv_tmp[i]=0;
      
      pb_trg[i]=0;      
     }
    }    
   } 
   
   if(dbuf[1]!=dv_tmp[1])
   {
    dbuf[1]=dv_tmp[1];
    set_airflow(dv_tmp[1]);
   }
   if(dbuf[0]!=dv_tmp[0])
   {
    dbuf[0]=dv_tmp[0];
    set_pow (dv_tmp[0]);
   }      
   
   if(!btn_trg && btn[0]==hold && btn[1]==hold)
   {
    btn_trg=1;
    dev_st = sdn;
   }
   
   
   if(dev_st!=work)
   {
    itr=0;
    write_ee_u8(dv_tmp[1], eep.airflow, sizeof(eep.airflow));  
    write_ee_u8(dv_tmp[0], eep.heatpow, sizeof(eep.heatpow)); 
   }
  }
  break; 
  
  case sdn:
  {
   if(!itr)
   {
    itr=1;
    set_pow(0);
    set_airflow(9);
    tim0 = 0xffff;
    dpp[1]=dpp[0]=0;
   }    

   if(!tim0)
   {
    set_airflow(0);
    dbuf[1]=dbuf[0]= '_';
   }
   else
   {
    if(!tim1)
    {
     tim1=1000;     
     convert_data(tim0/1000);
    }
   }
   
   if(!btn_trg)
   if(btn[0]==hold && btn[1]==hold) 
   {
    btn_trg=1;
    dev_st = start;
   }   
   
   if(dev_st!=sdn){itr=0;}
  }
  break; 
 }  
}