/*****************************************************
Workspace: CVAVR
Chip type: ATmega8 (internal RC_2MHz)
Project  : heat gun controller V3E
Date     : Dec, 2017
*****************************************************/
#include <mega8.h>
#include <stdlib.h>
#include <math.h>
#include <stdbool.h>
#include "defines.c"
#include "periph_init.c"
#include "7seg_routines.c"
#include "keywork.c"
#include "ee_work.c"
#include "adc_work.c"
#include "apow_work.c"
#include "t_work.c"

//volatile u16 test,test1,test2;
//volatile s16 stest,stest1,stest2;

u8 tim0,tim1,tim2,tim3;
bit kb_trg;

bit iwdg_t1c;




interrupt [TIM1_OVF]   void t1_ovf_isr(void)
{
 TCCR1A = 0;
 OC1A   = 0; 
 OC1B   = 0; 
}
interrupt [TIM1_CAPT]  void t1_capt_isr(void)
{
 if(ICR1<(u16)(mcu_freq/150))goto skip_cc1;

 TCNT1   = 0; 
 TCCR1B ^= 1<<ICES1;  
 TIFR    = 0xff;
   
 if(aBTA<0xff-10) 
 {
  OCR1A  = (u32)aBTA*ICR1>>8; 
  OC1A   = 1;
  TCCR1A|= A1|A0;
  TIMSK |= 1<<OCIE1A; 
 } 
 else          
 { 
  OC1A   = 0;
  TCCR1A&=~(A1|A0); 
 } 
  
 if(aZ01<0xff-10) 
 {
  OCR1B  = (u32)aZ01*ICR1>>8; 
  OC1B   = 1;
  TCCR1A&= ~(B1|B0);    
  TCCR1A|= B1; 
  OC1B   = 0;
  TIMSK |=1<<OCIE1B;
 } 
 else          
 {
  OC1B   = 1;
  TCCR1A&=~(B1|B0);  
 }    
 
 if(!adc_ready)
 {
  static u8 cnt=3; 
  
  cnt--;
  if(cnt==2) 
  {
   APOW_1; 
   ADCSRA = 1<<ADEN|1<<ADSC;
  }
  else if(cnt==1)
  {
   ADMUX  = 1<<REFS1|1<<REFS0; 
   ADCSRA = 1<<ADEN|1<<ADSC|1<<ADIE|5;
  } 
  else if(!cnt)
  {
   cnt = 48;
   ADMUX  = 1<<REFS1|1<<REFS0|1;
   ADCSRA = 1<<ADEN|1<<ADSC|1<<ADIE|5;   
  }
 }  

 if(tim0)tim0--;     
 if(tim1)tim1--;
 if(tim2)tim2--;
 if(tim3)tim3--; 
 if(kb_timer)kb_timer--;
 kb_trg=1; 
 iwdg_t1c=1;
 
 skip_cc1: 
}
interrupt [TIM1_COMPA] void t1_compa_isr(void)
{
 OCR1A += PW; 
 TCCR1A&= ~(A1|A0); 
 TCCR1A|= A0;  //toggle  
 TIMSK &=~(1<<OCIE1A);
}
interrupt [TIM1_COMPB] void t1_compb_isr(void)
{
 OCR1B += PW; 
 TCCR1A&= ~(B1|B0); 
 TCCR1A|= B0; //toggle  
 TIMSK &=~(1<<OCIE1B);
}
interrupt [ADC_INT]    void adc_isr()
{
 if(ADMUX & 1) 
 {
  adc_retval_1 = ADCW;
  ADMUX  = 0;
  ADCSRA = 0;
  APOW_0;   
  adc_ready = 1; 
 }
 else adc_retval_0 = ADCW;
}



void devcon(void);

void main(void)
{ 
 gpio_init();
 t1_init(); 
 ee_set(); 
 
 ss_pol=detect_7seg_polarity(); 
 
 #pragma optsize-
 WDTCR=0x1F;
 WDTCR=0x0F;
 #ifdef _OPTIMIZE_SIZE_
 #pragma optsize+
 #endif  

 #asm("sei")  
      
 endless()
 {   
  display_data();  
  
  if(kb_trg) 
  {
   kb_trg=0;
   scan_keys();
  }
  else
  {
   devcon();
  } 
  
  if(iwdg_t1c){iwdg_t1c=0; #asm("wdr");}
 } 
}

#pragma optsize+
void devcon(void)
{
 static bit itr=0;
 static enum {start,work,sdn,setup,err}dev_st = start;
 static bit btn_trg;
 static u8 pb_trg[3]; 
 static u8 airflow;
 static u8 heatpow;
 static u16 T_set;
 static s16 T_get;  
 static u8 errnum;
 static u8 err_cntdn;
 
 if(btn_trg && btn[2]==rls && btn[1]==rls && btn[0]==rls)btn_trg=0;



 switch (dev_st)
 {
  case start:
  {     
   static bit s_trg;
   if(!itr)
   {
    itr=1;
    tim0=0; 
    
    s_trg = read_ee(ee_onoff, sizeof(ee_onoff)); 
    if(s_trg)
    {    
     set_airflow(9);
     set_heatpow(100);
    } 
    dpp[2]=dpp[1]=dpp[0]=0;
   } 
  
   if(s_trg)
   {
    if(!tim0)   
    { 
     static s8 i=0; 
         
     tim0=25;   
         
     if(!i)       {dbuf[2]=dbuf[1]=dbuf[0]=10;}   
     else if(i<6) {dbuf[2]++;dbuf[1]++;dbuf[0]++;}   
     else i=-1,tim0=0; 
     i++;   
    } 
    
    if(!btn_trg)
    if(btn[2]!=rls || btn[1]!=rls || btn[0]!=rls)
    {
     dev_st=work;
     btn_trg=1; 
     i=0;
    }
   }
   else dev_st=sdn;      
     
   if(dev_st!=start){itr=0;}
  } 
  break;
  
  case work:
  {
   u8 i;  
   u16 tmp;
   
   if(!itr)
   {
    itr=1;

    airflow = read_ee(ee_airflow, sizeof(ee_airflow));
    T_set   = read_ee(ee_t, sizeof(ee_t))*10;
    
    set_airflow(airflow);
    set_heatpow((u16)airflow*100/9); 
    
    tim2=0xff;
    err_cntdn=0xff;
   } 
   
      
   if(get_T(&T_get)==1)
   {          
    static s16 T_data[TDN];   
    
    for(i=0;i<TDN-1;i++)T_data[i]=T_data[i+1];  //  
    T_data[TDN-1] = T_get;    
    
    //heat err detection
    if(T_data[TDN-1]<T_set && T_data[TDN-1]<T_data[TDN-2])
    {
     if(err_cntdn>1)err_cntdn-=2;
     else 
     {
      //dev_st = err;         
      errnum = 0; 
     }
    }
    else if(err_cntdn<0xff)err_cntdn++;
    //end of heat err detection
    
    heatpow = T_reg(T_set, T_data, airflow); 
    set_heatpow(heatpow);
    
    //data2display    
    if(tim1 && tim1>tim2) {convert_data(airflow);dbuf[2]='A';dbuf[1]='F';dpp[1]=1;}
    else if(tim2)         convert_data(T_set),dpp[1]=0;
    else                  convert_data(T_get),dpp[1]=0;
    //end of data2display   
   }   
   
   //keywork
   if(!btn_trg)
   {
    //key2set airflow   
    tmp = airflow;
    if(btn[AF]==prs) pb_trg[AF]=1;
    else if(btn[AF]==hold)
    {
     pb_trg[AF]=0;

     if(!tim0)
     {
      tim0=25;
      if(airflow<9)airflow++;
      else airflow=3;      
     }
    }
    else
    {
     if(pb_trg[AF])
     { 
      pb_trg[AF]=0;
      
      if(airflow<9)airflow++;
      else airflow=3;
     }
    }    
   
    if(tmp!=airflow) tim1=0xff;
    //end of key2set airflow

    //keys2set T
    tmp = T_set;   
    kb_input(&T_set,0, 600, 10, 10, 25);
   
    if(tmp!=T_set) tim2=0xff;
   //end of keys2set T
   
    //switch_mode              
    tmp=0;
    for(i=0;i<3;i++)if(btn[i]==hold)tmp++;
    if(tmp>2)
    {
     btn_trg=1;
     dev_st = setup;
    } 
    else if(tmp>1) 
    {
     btn_trg=1;
     dev_st = sdn;    
    } 
    //end of swith_mode    
   } 
   //end of keywork
   
   
   if(dev_st!=work)
   {
    itr=0;
   }
  }
  break;  
    
  case sdn:
  { 
   static u8 cnt;
   if(!itr)
   {
    itr=1; 
    
    if(read_ee(ee_onoff, sizeof(ee_onoff))==1)
    {
     cnt=0x0b;   
     
     set_airflow(9);
     set_heatpow(0); 
     
     write_ee(0, ee_onoff, sizeof(ee_onoff));
     write_ee(airflow, ee_airflow, sizeof(ee_airflow));
     write_ee((u8)(T_set/10), ee_t, sizeof(ee_t));      
    }
    else 
    {
     tim0=0;
     cnt=0;  
     
     set_airflow(0);
     set_heatpow(0);      
    }
   }  
 
   if(!tim0)
   {   
    if(cnt)
    {
     cnt--; 
     tim0=0xff;
    } 
   }     
   
   if(cnt || tim0)
   {
    if(!tim1)
    {
     tim1 = 100;
     convert_data(((s16)cnt*0xff + tim0)/100);
    }
   }
   else
   {      
    set_airflow(0);
    dbuf[2]=dbuf[1]=dbuf[0]='_'; 
   } 

   if(!btn_trg) 
   { 
    u8 i;
    u8 tmp=0;
    for(i=0;i<3;i++)if(btn[i]==hold)tmp++;    
   
    if(tmp>1) 
    {
     btn_trg=1;
     dev_st = start; 
     write_ee(1, ee_onoff, sizeof(ee_onoff));
    }   
   }
   
   if(dev_st!=sdn){itr=0;}
  }  
  break;
      
  case setup:
  {      
   static enum {adj_T_dd, adj_T_ofs} se;
   static u16 T_diode;
   static u16 T_offset; 
   static bit mtrg;
   if(!itr)
   {        
    itr=1; 
    
    set_airflow(9);
    set_heatpow(0);
               
    T_diode = ee_T20_d/100;
    T_offset = ee_T_offset/100;
    
    se = adj_T_dd;
    dpp[2]=1; 
    convert_data(T_diode); 
    dbuf[2]='d';
   } 
        
   if(btn[AF]==rls)mtrg=0; 
   
   if(!mtrg)
   {
    u16 tmp; 
    
    if(btn[AF]==prs) 
    {
     mtrg=1;
     
     if(se==adj_T_dd)
     {
      se = adj_T_ofs;
      convert_data(T_offset); 
      dbuf[2]='t'; 
     } 
     else  
     {
      se = adj_T_dd;
      convert_data(T_diode); 
      dbuf[2]='d'; 
     }      
    }  
    
    if(se==adj_T_dd)
    { 
     tmp = T_diode;
     kb_input(&T_diode, 0, 60, 1, 5, 0xff); 
     if(tmp!=T_diode) 
     {
      convert_data(T_diode); 
      dbuf[2]='d'; 
     }
    } 
    else  
    {   
     tmp = T_offset;
     kb_input(&T_offset, 0, 60, 1, 5, 0xff); 
     if(tmp!=T_offset) 
     {
      convert_data(T_offset); 
      dbuf[2]='t'; 
     }     
    }   
   } 
   
   if(!btn_trg)
   {
    if(btn[AF]==hold)
    {   
     btn_trg=1;
     dev_st = work;
    }
   }
   
   
   if(dev_st!=setup)
   {     
    itr=0;
    dpp[2]=0;  
           
    if(ee_T20_d != T_diode*100)
    {      
     bool tmp_r;
     u16 tmp_r1,tmp_r0;
    
     ee_T20_d = T_diode*100;
     do{tmp_r = read_adc(&tmp_r0, &tmp_r1);}while(!tmp_r); 
     if(ee_V20_d != tmp_r1)ee_V20_d = tmp_r1;  
    }
    
    if(T_offset*100 != ee_T_offset) ee_T_offset = T_offset*100;
    dev_st = work;
   }
  }
  break;
     
  case err:
  {
   if(!itr)
   {
    itr=1;

    dbuf[2]='E';dbuf[1]=' ';dbuf[0]=errnum; 
    dpp[1]=1;
   }
   
   if(dev_st!=err){itr=0;}
  }   
  break;
 } 
}
#pragma optsize-
